; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "Fresh artistic license."         |
;|_______________________________________________________________________________________|
;                                                                                        |
;   Description: Test program for TFreshEdit control.                                    |
;                                                                                        |
;   Target OS: All/Linux/Win32 - depending on current test needs.                        |
;                                                                                        |
;   Dependencies:                                                                        |
;                                                                                        |
;   Notes: The main test program calls "PrepareDemo" procedure.                          |
;________________________________________________________________________________________|


__TestGround:

; Global data...
iglobal

__TestData:

frmMainForm:
        ObjTemplate  tfParent or tfEnd, Form, frmMain, \
                     x, 100,        \
                     y, 50,         \
                     width, 640,    \
                     height, 480,   \
                     borderKind, borderFull,    \
                     Visible, TRUE, \
                     Caption, 'FreshEdit test application.'

        ObjTemplate  tfParent, Form, Toolbar,           \
                     x, 0, y, 0,                        \
                     width, 100, height, 20,            \
                     Align, waBottom,                   \
                     borderKind, borderNone,         \
                     Visible, TRUE

        ObjTemplate  tfChild, Button, btnOpen,  \
                     x, 112, y, 2,                        \
                     width, 64, height, 21,            \
                     Align, waLeft,                     \
                     Caption, 'Open',                   \
                     TextAlign, dtfAlignCenter or dtfAlignMiddle,       \
                     OnClick, OnOpenFile,         \
                     Visible, TRUE

        ObjTemplate  tfChild, Button, btnSave,  \
                     x, 180, y, 2,                     \
                     width, 64, height, 21,            \
                     Align, waLeft,                     \
                     Caption, 'Save',                   \
                     TextAlign, dtfAlignCenter or dtfAlignMiddle,       \
                     OnClick, OnSaveFile,         \
                     Visible, TRUE

        ObjTemplate  tfChild, Button, btnNumbers,  \
                     x, 180, y, 2,                     \
                     width, 21, height, 21,            \
                     Align, waLeft,                     \
                     Caption, '#',                  \
                     TextAlign, dtfAlignCenter or dtfAlignMiddle,       \
                     OnClick, OnNumbers,         \
                     Visible, TRUE

        ObjTemplate  tfChild, Button, btnZebra,  \
                     x, 180, y, 2,                     \
                     width, 21, height, 21,            \
                     Align, waLeft,                     \
                     Caption, 'Z',                  \
                     TextAlign, dtfAlignCenter or dtfAlignMiddle,       \
                     OnClick, OnZebra,         \
                     Visible, TRUE

        ObjTemplate  tfChild, Button, btnTheme,  \
                     x, 180, y, 2,                     \
                     width, 21, height, 21,            \
                     Align, waLeft,                     \
                     Caption, 'T',                  \
                     TextAlign, dtfAlignCenter or dtfAlignMiddle,       \
                     OnClick, OnTheme,         \
                     Visible, TRUE

        ObjTemplate  tfEnd, Edit, FilenameEdit,         \
                     x, 2, y, 2,                        \
                     width, 100, height, 21,            \
                     Align, waClient,                     \
                     Visible, TRUE

        ObjTemplate tfEnd, FreshEdit, Editor,   \
                    x, 8, y, 8,                 \
                    width, 624, height, 464,    \
                    Align, waClient,            \
                    Visible, TRUE


  SampleText file 'unicode_test.txt'
             dd   0

__TestDataSize = $ - __TestData
endg


uglobal
  Timer1 dd ?
endg

var FreshEditSetTheme = TFETheme.SetDefaultThemeWindows


proc OnTheme, .self, .mousebtn
begin
        xor     [FreshEditSetTheme], TFETheme.SetDefaultThemeWindows xor TFETheme.SetDefaultThemeClassic
        stdcall [FreshEditSetTheme], FreshEditTheme
        xor     [SynTheme], fasm_colors_classic xor fasm_colors_windows
        execute [Editor], TFreshEdit.Refresh
        return
endp


proc OnZebra, .self, .mousebtn
begin
        xor     [FreshEditTheme.Options], eoStripedBackground
        execute [Editor], TFreshEdit.Refresh
        return
endp


proc OnNumbers, .self, .mousebtn
begin
        xor     [FreshEditTheme.Options], eoLineNumbers
        execute [Editor], TFreshEdit.Refresh
        return
endp


cErrorTitleOpen text "Error open file."
cErrorTitleSave text "Error save file."

proc OnSaveFile, .self, .mousebtn
begin
        pushad

        stdcall Get, [FilenameEdit], TEdit.Text
        push    eax
        stdcall StrPtr, eax
        mov     edi, eax

        stdcall Get, [Editor], TFreshEdit.Text
        jc      .finish

        push    eax
        stdcall StrPtr, eax

        stdcall SaveBinaryFile, edi, eax, [eax+string.len]
        jnc     .end_save

        stdcall GetErrorString, eax
        mov     esi, eax
        stdcall ShowMessage, [frmMain], smiError, cErrorTitleSave, esi, smbOK
        stdcall FreeErrorString, esi

.end_save:
        stdcall StrDel ; from the stack
.finish:
        stdcall StrDel ; from the stack
        popad
        return
endp


proc OnOpenFile, .self, .mousebtn
begin
        pushad
; Load test file
        stdcall Get, [FilenameEdit], TEdit.Text
        push    eax

        stdcall StrPtr, eax
        stdcall LoadBinaryFile, eax
        jnc     .loaded

        stdcall GetErrorString, eax
        mov     esi, eax
        stdcall ShowMessage, [frmMain], smiError, cErrorTitleOpen, esi, smbOK
        stdcall FreeErrorString, esi
        jmp     .exit

.loaded:
        push    eax
        stdcall Set, [Editor], TFreshEdit.Text, eax
        stdcall FreeMem ; from the stack

.exit:
        stdcall StrDel ; from the stack

        mov     eax, [Editor]
        mov     [eax+TFreshEdit._xCaret], 0
        mov     [eax+TFreshEdit._yCaret], 0
        mov     [eax+TFreshEdit._xSelection], 0
        mov     [eax+TFreshEdit._ySelection], 0

;        stdcall ParseFolding, [eax+TFreshEdit._pLines]

        execute [Editor], TFreshEdit.Refresh
        execute [Editor], TFreshEdit.Focus
        popad
        return
endp



proc PrepareDemo
begin
        stdcall CreateFromTemplate, frmMainForm, 0
        mov     ecx, [Application]
        mov     [ecx+TApplication.MainWindow], frmMain

        stdcall Set, [Editor], TFreshEdit._procSyntax, SyntaxFASM
        stdcall Set, [Editor], TFreshEdit.OnControlKey, procKeyPress
        stdcall Set, [Editor], TFreshEdit.Text, SampleText

        mov     esi, [Editor]
; test - set some flags on line 33
        mov     eax, [esi+TFreshEdit._pLines]
        or      [eax+TArray.array+sizeof.TEditorLine*26+TEditorLine.flags], lfBreakpoint

; test - set some flags on line 34
        mov     eax, [esi+TFreshEdit._pLines]
        or      [eax+TArray.array+sizeof.TEditorLine*27+TEditorLine.flags], lfBreakpoint
        mov     [eax+TArray.array+sizeof.TEditorLine*27+TEditorLine.debugdata], $12345
        mov     [eax+TArray.array+sizeof.TEditorLine*28+TEditorLine.debugdata], $12345
        mov     [eax+TArray.array+sizeof.TEditorLine*29+TEditorLine.debugdata], $12345

        stdcall ParseFolding, eax
.exit:
        return
endp



proc procKeyPress, .self, .keyevent
begin
        mov     ebx, [.keyevent]

        mov     eax, [ebx+TKeyboardEvent.key]

        cmp     eax, $03        ; ctrl+C
        je      .copy
        cmp     eax, $16        ; ctrl+V
        je      .paste
        cmp     eax, 12 ; ctrl+L
        je      .length_list
        cmp     eax, 18 ; ctrl+R
        je      .clipright
        cmp     eax, 19 ; ctrl+S == save
        je      .save
        cmp     eax, 20 ; ctrl+T
        je      .test_something

        cmp     [ebx+TKeyboardEvent.scancode], keyF2
        je      .toggle_breakpoint

.finish:
        mov     eax, chgfNoClearSelection
.exit:
        return

.copy:
        stdcall Get, [.self], TFreshEdit.Selection
        test    eax, eax
        jz      .end_copy
        stdcall ClipboardWrite, eax
        stdcall StrDel, eax
.end_copy:
        mov     eax, chgfNoClearSelection
        return

.paste:
        stdcall ClipboardRead
        test    eax, eax
        jz      .end_paste

        stdcall Set, [.self], TFreshEdit.Selection, eax
        stdcall StrDel, eax

        mov     eax, chgfNeedRefresh
.end_paste:
        return


.save:
        stdcall OnSaveFile, [.self], 0
        xor     eax, eax
        return

.length_list:
        xor     eax, eax
        return

.clipright:
        stdcall Get, esi, TFreshEdit.CurrentLine
        jc      .finish
        stdcall StrClipSpacesR, [eax+TEditorLine.Data]
        mov     eax, chgfNeedRefresh
        return

.test_something:
        mov     eax, chgfNeedRefresh
        return


.toggle_breakpoint:
        cmp     [esi+TFreshEdit._fReadOnly], froReadWrite
        jne     .finish

        stdcall Get, esi, TFreshEdit.CurrentLine
        jc      .finish

        xor     [eax+TEditorLine.flags], lfBreakpoint
        mov     eax, chgfNeedRefresh or chgfNoClearSelection
        return


;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

endp




; very primitive! To be rewrited from scratch to allow custom keywords.
proc ParseFolding, .pLines
begin
        pushad

        xor     ebx, ebx
        mov     esi, [.pLines]
        mov     ecx, [esi+TArray.count]
        lea     esi, [esi+TArray.array]

.loop:
        jecxz   .endparse

        stdcall StrPtr, [esi+TEditorLine.Data]
        jc      .forward

        cmp     [eax+string.len], 4
        jb      .forward

        cmp     dword [eax], 'proc'
        je      .up

        cmp     [eax+string.len], 6
        jb      .forward

        cmp     dword [eax], 'loca'
        jne     .forward
        cmp     word [eax+4], 'ls'
        jne     .forward

.up:
        inc     ebx
        or      [esi+TEditorLine.flags], lfFoldHeader

.forward:
        mov     [esi+TEditorLine.fold_level], ebx

        stdcall StrPtr, [esi+TEditorLine.Data]
        jc      .forward2

        cmp     [eax+string.len], 4
        jb      .forward2

        cmp     dword [eax], 'endp'
        je      .down
        cmp     dword [eax], 'endl'
        jne     .forward2

.down:
        test    ebx, ebx
        jz      .forward2
        dec     ebx

.forward2:
        add     esi, sizeof.TEditorLine
        dec     ecx
        jmp     .loop

.endparse:
        popad
        return
endp








DispSize 'Test ground dummy code:', ($-__TestGround)+__TestDataSize

; without this, the application crashes in Linux with untraceable seg fault. Why???
; It seems it crashes during the load, before even one instruction executed.
; Maybe it is some kind of bug in the elf formater...
; IT CRASHES ONLY SOMETIMES?????
;rb 64

include 'FreshEdit/fasm_syntax.asm'
