; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "Fresh artistic license."         |
;|_______________________________________________________________________________________|
;
;  Description: The main file of FreshLib test application. Main file.
;
;  Target OS: Any
;
;  Dependencies: FreshLib - see includes in the source.
;
;  Notes: This is the main file of the project. It forms the common frame of the GUI
;         program. The real test code is separated in TestGroundComplex.asm or
;         TestGround.asm files.
;_________________________________________________________________________________________

include 'compiler/executable.inc'
_BinaryType GUI

include 'macros/allmacros.inc'
include 'equates/allequates.inc'


_CodeSection

options.FastEnter         = 0           ; enter/leave or push ebp/pop ebp approach to the procedures.
options.ShowSkipped       = 0           ; shows the procedures skiped because of no use.
options.ShowSizes         = 1           ; enable/disable work of DispSize macro.
options.DebugMode         = 0           ; enable/disable macroses of simpledebug library.



include 'simpledebug/debug.asm'

include 'GUI/SysEvents.asm'
include 'system/process.asm'
include 'system/memory.asm'
include 'system/files.asm'
include 'system/clipboard.asm'
include 'data/arrays.asm'
include 'data/strlib.asm'
include 'data/uConfig.asm'

include 'timers/timers.asm'

include 'graphics/context.asm'
include 'graphics/text.asm'
include 'graphics/draw.asm'
include 'graphics/images.asm'
include 'graphics/backbuffer.asm'
include 'graphics/giflib.asm'
include 'graphics/fonts.asm'

include 'GUI/GUI.asm'
include 'FreshEdit/FreshEdit.asm'
include 'mouse/mouse.asm'
include 'dialogs/dialogs.asm'



; use only one of the following files.
;
;include 'TestGround.asm'
;include 'TestGroundComplex.asm'
;include 'TestGroundKeyboard.asm'
include 'TestGroundFreshEdit.asm'
;include 'TestGroundConfig.asm'


;if TargetOS eq Linux
;  rb 604       ; this bytes simply make ELF file to be loadable.
;              ; there is a bug somewhere in the ELF generator (FASM or possibly ELF macros)
;              ; that makes some sizes of the ELF executable not loadable. (the Linux loader crashes with SEGFAULT)
;end if
;


; Main Program
start:
DebugMsg 'Before initializing.'

        InitializeAll

DebugMsg 'After initializing.'

        stdcall Create, CApplication
        jc      .start_error
        mov     [Application], ebx

        stdcall PrepareDemo


DebugMsg 'Just before run.'

        stdcall Run

.terminate:
        push    eax
        FinalizeAll

        DebugMsg 'Just before the end.'
        stdcall Terminate       ; from the stack

.start_error:
        mov     eax, 1
        DebugMsg "Can't create application object."
        jmp      .terminate

; end of main program

_DataSection
IncludeAllGlobals


_ImportSection
include 'imports/allimports.asm'


