; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "Fresh artistic license."         |
;|_______________________________________________________________________________________|
;
;  Description: Files manipulation library. Linux part.
;
;  Target OS: Linux
;
;  Dependencies: Uses system calls for file processing and memory.asm for memory
;                allocations.
;
;  Notes:
;_________________________________________________________________________________________

fsFromBegin     = 0
fsFromEnd       = 2
fsFromCurrent   = 1

sys_read        = 03h
sys_write       = 04h
sys_open        = 05h
sys_close       = 06h
sys_unlink      = 0Ah
sys_lseek       = 013h

;   file access bits:
O_ACCMODE       = 0003o
O_RDONLY        = 00o
O_WRONLY        = 01o
O_RDWR          = 02o
O_CREAT         = 0100o
O_EXCL          = 0200o
O_NOCTTY        = 0400o
O_TRUNC         = 01000o
O_APPEND        = 02000o
O_NONBLOCK      = 04000o
O_NDELAY        = O_NONBLOCK
O_SYNC          = 010000o ;specific to ext2 fs and block devices
FASYNC          = 020000o ;fcntl, for BSD compatibility
O_DIRECT        = 040000o ;direct disk access hint - currently ignored
O_LARGEFILE     = 0100000o
O_DIRECTORY     = 0200000o ;must be a directory
O_NOFOLLOW      = 0400000o ;don't follow links

;   file permissions flags
S_ISUID         = 04000o ;set user ID on execution
S_ISGID         = 02000o ;set group ID on execution
S_ISVTX         = 01000o ;sticky bit
S_IRUSR         = 00400o ;read by owner (S_IREAD)
S_IWUSR         = 00200o ;write by owner (S_IWRITE)
S_IXUSR         = 00100o ;execute/search by owner (S_IEXEC)
S_IRGRP         = 00040o ;read by group
S_IWGRP         = 00020o ;write by group
S_IXGRP         = 00010o ;execute/search by group
S_IROTH         = 00004o ;read by others (R_OK)
S_IWOTH         = 00002o ;write by others (W_OK)
S_IXOTH         = 00001o ;execute/search by others (X_OK)

;--------------------------------------
; if CF = 0, eax is handle to the file.
; if CF = 1, eax is error code.
;--------------------------------------
proc FileOpen, .filename
begin
        push    edx ecx ebx
        mov     eax, sys_open
        mov     ebx, [.filename]
        mov     ecx, O_RDWR
        mov     edx, S_IWUSR+S_IWGRP+S_IWOTH
        int     0x80
        cmp     eax, -1
        jle     .error
        clc
        pop     ebx ecx edx
        return

.error:
        neg     eax
        DebugMsg "Error open file. Error code:"
        OutputRegister regEAX, 16
        stc
        pop     ebx ecx edx
        return
endp



;--------------------------------------
; if CF = 0, eax is handle to the file.
; if CF = 1, eax is error code.
;--------------------------------------
proc FileCreate, .filename
begin
        push    edx ecx ebx
        mov     eax, sys_open
        mov     ebx, [.filename]
        mov     ecx, O_CREAT+O_TRUNC+O_RDWR
        mov     edx, S_IRUSR+S_IWUSR+S_IRGRP+S_IROTH
        int     0x80
        cmp     eax, -1
        jle     .error
        clc
        pop     ebx ecx edx
        return
.error:
        neg     eax
        stc
        pop     ebx ecx edx
        return
endp




;--------------------------------------
; if CF = 0, the file was closed.
; if CF = 1, eax is error code.
;--------------------------------------
proc FileClose, .handle
begin
        push    ebx
        mov     eax, sys_close
        mov     ebx, [.handle]
        int     0x80
        cmp     eax, -1
        jle     .error
        clc
        pop     ebx
        return
.error:
        neg     eax
        stc
        pop     ebx
        return
endp




proc FileRead, .handle, .buffer, .count
begin
        push    edx ecx ebx
        mov     eax, sys_read
        mov     ebx, [.handle]
        mov     ecx, [.buffer]
        mov     edx, [.count]
        int     0x80
        cmp     eax, -1
        jle     .error
        clc
        pop     ebx ecx edx
        return

.error:
        neg     eax
        stc
        pop     ebx ecx edx
        return
endp


proc FileWrite, .handle, .buffer, .count
begin
        push    edx ecx ebx
        mov     eax, sys_write
        mov     ebx, [.handle]
        mov     ecx, [.buffer]
        mov     edx, [.count]
        int     0x80
        cmp     eax, -1
        jle     .error
        clc
        pop     ebx ecx edx
        return

.error:
        neg     eax
        stc
        pop     ebx ecx edx
        return
endp


;----------------------------------------------------
; if CF = 0, eax is the new possition in the file
; if CF = 1, eax is error code.
;----------------------------------------------------
proc FileSeek, .handle, .dist, .direction
begin
        push    edx ecx ebx
        mov     eax,sys_lseek
        mov     ebx,[.handle]
        mov     ecx,[.dist]
        mov     edx,[.direction]
        int     0x80
        cmp     eax,-1
        jle     .error
        clc
        pop     ebx ecx edx
        return

.error:
        neg     eax
        stc
        pop     ebx ecx edx
        return
endp



;--------------------------------------
; if CF = 0, the file was deleted.
; if CF = 1, eax is error code.
;--------------------------------------
proc FileDelete, .filename
begin
        push    ebx
        mov     eax,sys_unlink
        mov     ebx,[.filename]
        int     0x80
        cmp     eax,-1
        jle     .error
        clc
        pop     ebx
        return

.error:
        neg     eax
        stc
        pop     ebx
        return
endp



proc GetErrorString, .code
.buff dd ?
begin
        push    ecx edx
        cinvoke strerror_r, [.code], 0, 0       ; it is somewhat not clear about the returned string. Should we free it???
        pop     edx ecx
        clc
        return
endp




proc FreeErrorString, .ptrString
begin
        return
endp
