; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "Fresh artistic license."         |
;|_______________________________________________________________________________________|
;
;  Description: Process management library. Linux part
;
;  Target OS: Linux
;
;  Dependencies: Uses pthreads library for thread creation.
;
;  Notes:
;_________________________________________________________________________________________


; Terminate current process
; Returns: eax = 0
proc Terminate,.exit_code
begin
        mov     eax, sys_exit
        mov     ebx, [.exit_code]
        int     0x80
        return
endp



proc ThreadCreate, .ptrFunction, .ptrArguments
.threadid dd ?
begin
        push    ecx edx
        lea     eax, [.threadid]
        mov     [.threadid], 0
        cinvoke pthread_create, eax, NULL, [.ptrFunction], [.ptrArguments]
        mov     eax, [.threadid]
        test    eax, eax
        jz      .error

        clc
        pop     edx ecx
        return

.error:
        stc
        pop     edx ecx
        return
endp




; MUTEX functions
;_________________________________________________________________________________________

proc MutexCreate, .ptrName, .ptrMutex
begin
        push    eax ecx edx
        pop     edx ecx eax
        return
endp




proc WaitForMutex, .ptrMutex, .timeout
begin
        push    eax ecx edx
        clc
        pop     edx ecx eax
        return
endp




proc MutexRelease, .ptrMutex
begin
        pushf
        push    eax ecx edx
        pop     edx ecx eax
        popf
        return
endp



proc MutexDestroy, .ptrMutex
begin
        push    eax ecx edx
        pop     edx ecx eax
        return
endp









;__ThreadStackSize = 1024 * 1024  ; 1Mbyte for the thread stack
;
;; Creates a new execution thread
;; Returns: CF=1 if error.
;;          CF=0, eax = pid of the new process being created
;proc ThreadCreate, .ptr_to_function, .ptr_to_args
;begin
;        push    ebx ecx edx esi edi
;
;        push    ebp
;        mov     eax, sys_mmap_pgoff
;        xor     ebx, ebx
;        mov     ecx, __ThreadStackSize
;        mov     edx, PROT_READ or PROT_WRITE
;        mov     esi, MAP_ANONYMOUS or MAP_PRIVATE or MAP_GROWSDOWN or MAP_STACK
;        mov     edi, -1
;        xor     ebp, ebp
;        int $80
;        pop     ebp
;
;        cmp     eax, $ffffff00
;        jae     .error
;
;        lea     ecx, [eax+__ThreadStackSize-8]
;
;; transfer arguments in the new stack.
;        mov     eax, [.ptr_to_function]
;        mov     [ecx], eax
;        mov     eax, [.ptr_to_args]
;        mov     [ecx+4], eax
;
;        mov     eax, sys_clone
;        mov     ebx, CLONE_VM or CLONE_FILES or CLONE_SIGHAND or CLONE_IO or CLONE_THREAD
;        int 0x80
;
;        cmp     eax,-1
;        je      .error
;
;        test    eax, eax
;        jz      .is_clone
;
;        clc
;        pop     edi esi edx ecx ebx
;        return
;
;; The stack is not allocated or sys_clone exits with error.
;.error:
;        stc
;        pop     edi esi edx ecx ebx
;        return
;
;; this is the clone process - call the thread.
;.is_clone:
;        pop     eax
;        call    eax       ; the argument is already in the stack.
;        mov     esi, eax  ; the thread procedure can return exit code. Save it in esi.
;
;; release the stack.
;        mov     eax, sys_munmap
;        lea     ebx, [esp-__ThreadStackSize]
;        mov     ecx, __ThreadStackSize
;        int $80
;
;; and exit the process  - the question is - how to execute int $80 without a stack ??? However, by some magic, it works...
;        mov     ebx, esi
;        mov     eax, sys_exit
;        int     0x80
;endp
;