; _______________________________________________________________________________________
;|                                                                                       |
;| ..::FreshLib::..  Free, open source. Licensed under "Fresh artistic license."         |
;|_______________________________________________________________________________________|
;
;  Description: Files manipulation library.
;
;  Target OS: Any
;
;  Dependencies: Uses memory.asm for memory allocations.
;
;  Notes: This file contains OS independent part of the library and also include
;         the respective OS dependent part.
;_________________________________________________________________________________________
module "Files library"

include '%TargetOS%/files.asm'


;--------------------------------------------------------------------
; Allocates needed memory and loads the entire file in this memory.
; Returns pointer to memory block or NULL if there is error.
; Returns size in ecx
;--------------------------------------------------------------------
proc LoadBinaryFile, .ptrFileName
begin
        push    esi edi ebx

        stdcall FileOpen, [.ptrFileName]
        jc      .error

        mov     edi, eax

        stdcall FileSeek, edi, 0, fsFromEnd
        jc      .errorclose
        mov     ebx, eax

        lea     ecx, [ebx+8]    ; It is for the case that the file is
                                ; text and we want to use it as null terminated
                                ; string.
        stdcall GetMem, ecx
        jc      .errorclose

        mov     esi, eax
        stdcall FileSeek, edi, 0, fsFromBegin
        jc      .errorclosemem
        stdcall FileRead, edi, esi, ebx
        jc      .errorclosemem

        mov     ecx, eax

        stdcall FileClose, edi

        mov     eax, esi
        clc
        pop     ebx edi esi
        return


.errorclosemem:
        push    eax
        stdcall FreeMem, esi
        pop     eax

.errorclose:
        push    eax
        stdcall FileClose, edi
        pop     eax

.error:
        xor     ecx, ecx
        stc
        pop     ebx edi esi
        return
endp



;--------------------------------------------------------------------
; Saves to the file [size] bytes from given [ptr] address.
; returns CF=1 if error and CF=0 if OK.
;--------------------------------------------------------------------
proc SaveBinaryFile, .ptrFileName, .aptr, .size
begin
        push    edi

        stdcall FileCreate, [.ptrFileName]
        jc      .error

        mov     edi, eax
        stdcall FileWrite, edi, [.aptr], [.size]
        jc      .errorclose

        push    eax
        stdcall FileClose, edi
        pop     eax

        clc
        pop     edi
        return

.errorclose:
        push    eax
        stdcall FileClose, edi
        pop     eax

.error:
        stc
        pop     edi
        return
endp



; Returns:
; CF=1 if the file NOT EXISTS
; CF=0 if the file EXISTS

proc FileExists, .ptrFileName
begin
        push    eax

        push    [.ptrFileName]
        stdcall FileOpen
        jc      .exit
        stdcall FileClose, eax
        clc
.exit:
        pop     eax
        return
endp


endmodule