uglobal
  hEditorMenu     dd 0                        ; source popup menu item.
  hBookmarkIcon   dd 0
  CurrentSettings dd  ?
endg


iglobal

defaultSettings:
Theme                                                                   \
  'Fixedsys', -12, DEFAULT_CHARSET,                                     \
  AES_AUTOINDENT or AES_SMARTTABS or AES_CONSOLECARET,                  \
  $FFFFFF, $800000, $000000, $C0C0C0, $C0C0C0, $C0C0C0, 20, 4,          \
  $000000, $ffff00,                                                     \
\; syntax colors
  $D0D0D0, $FFFF00, $FFFF00, $A0A0A0, $00FF00, $00FFFF, $ff00ff, $ff00ff


CoolMenu EditorPopup,                   \
    mfNormal, actGotoDefinition,        \
    mfNormal, actCrossReference,        \
    mfNormal, actOpenFileAtCursor,      \
    mfNormal, actInclude,               \
    mfSeparator, NONE,                  \
    mfNormal, actCut,                   \
    mfNormal, actCopy,                  \
    mfNormal, actPaste,                 \
    mfSeparator, NONE,                  \
    mfNormal,    actIndent,             \
    mfNormal,    actOutdent,            \
    mfSeparator, NONE,                  \
    mfNormal,    actComment,            \
    mfNormal,    actUncomment,          \
    mfSeparator, NONE,                  \
    mfSubmenu, '&Bookmarks',            \
      mfNormal,    actToggleBM,         \
      mfSeparator, NONE,                \
      mfNormal,    actNextBM,           \
      mfNormal,    actPrevBM,           \
      mfSeparator, NONE,                \
      mfNormal,    actClearBM,          \
    mfEnd, END
endg


CCM_UPDATECCPOS = WM_USER+$f000
CCM_UPDATEPAPOS = WM_USER+$f001

cCurrentEditorSettings text 'Current.theme'

;----------------------------------------
; Creates sourceeditor window.
; Call this only once.
;----------------------------------------
proc InitSourceEditor
begin
        stdcall CreateCoolMenu, EditorPopup, TRUE
        mov     [hEditorMenu], eax

        invoke  LoadImage, [hInstance], resBookmarkIcon, IMAGE_ICON, 0, 0, LR_SHARED
        mov     [hBookmarkIcon], eax

; Load editor color profile...
        stdcall LoadEditorSettings, cCurrentEditorSettings
        jnc     .profileok

        stdcall ThemeDuplicate, defaultSettings

.profileok:
        mov     [CurrentSettings], eax
        return
endp


proc FinalizeSourceEditor
begin
        stdcall SaveEditorSettings, cCurrentEditorSettings, [CurrentSettings], TRUE
        stdcall DestroySettings, [CurrentSettings]

        invoke DestroyMenu, [hEditorMenu]
        invoke DestroyIcon, [hBookmarkIcon]
        return
endp


;--------------------------------------------------------
; Creates the editor window and returns handle to it.
;--------------------------------------------------------
proc CreateSourceEditor, .ptrOpenFile, .hTemplate
begin
        push    ebx esi edi

        mov     esi, [.ptrOpenFile]
        mov     edi, [CurrentSettings]

        xor     eax, eax
        mov     ecx, WS_CHILD or WS_HSCROLL or WS_VSCROLL or ES_NOHIDESEL or ES_WANTRETURN or WS_CLIPCHILDREN
        or      ecx, [edi+TEditorSettings.AsmEditStyle]

        invoke  CreateWindowEx, WS_EX_CLIENTEDGE or WS_EX_ACCEPTFILES, cAsmEditClassName,    \
                eax, ecx, eax, eax, eax, eax, [hEditorsHost], eax,      \
                [hInstance], eax

        test    eax, eax
        jz      .errorcreate

        mov     ebx, eax
        mov     [esi+TOpenFile.hEditor], ebx

        invoke  SetWindowLong, ebx, ofsOpenFileStruct, esi

; adjust the look and feel for the editor window.
        invoke  CreateFont, [edi+TEditorSettings.FontSize], 0, 0, 0, FW_NORMAL,                 \
                            0, 0, 0, [edi+TEditorSettings.FontCharset],                         \
                            OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY,           \
                            FIXED_PITCH or FF_DONTCARE, [edi+TEditorSettings.ptrFontName]

        invoke  SendMessage, ebx, WM_SETFONT, eax, FALSE
        invoke  SendMessage, ebx, AEM_SETTHEME, 0, [edi+TEditorSettings.ptrTheme]
        mov     eax, [esi+TOpenFile.ptrType]
        invoke  SendMessage, ebx, AEM_SETSYNTAXHIGHLIGHT, 0, [eax+TFileType.lParam]
        invoke  SendMessage, ebx, AEM_SETPOPUPMENU, 0, [hEditorMenu]
        invoke  SendMessage, ebx, AEM_SETBOOKMARKICON, [hBookmarkIcon], 0

        stdcall SubclassWindow, ebx, SourceEditProc
        invoke  SendMessage, ebx, CEM_LOADFILE, [.hTemplate], 0
        test    eax, eax
        jnz     .errorload

        invoke  SendMessage, [hEditorsHost], EHM_DOCKWINDOW, ebx, dfClient
        xor     eax, eax
        pop     edi esi ebx
        return

.errorcreate:
        invoke  GetLastError
        pop     edi esi ebx
        return

.errorload:
        push    eax
        invoke  DestroyWindow, ebx
        pop     eax
        pop     edi esi ebx
        return
endp





winproc SourceEditProc
.aepos AEPOS
.str   rb 256
.textlen dd ?
.written dd ?
.memoryfile TInMemoryFile
begin
        invoke  GetWindowLong, [.hwnd], ofsOpenFileStruct
        mov     edi, eax

ondefault
        stc
        return


;------- Custom message handlers ------------------

onmessage WM_DESTROY
        stdcall ClosePopup, [hCCList], 0, 0
        stdcall ClosePopup, [hPAHint], 0, 0
        stc
        return


onmessage _CEM_GETTEXT
        mov     ebx, [.lparam]   ; pointer to TInMemoryFile

        invoke  SendMessage, [.hwnd], WM_GETTEXTLENGTH, 0, 0
        mov     [ebx+TInMemoryFile.fileSize], eax
        lea     esi, [eax+1]

        stdcall GetMem, esi
        jc      .errorgettext

        mov     [ebx+TInMemoryFile.ptrBuffer], eax
        mov     [ebx+TInMemoryFile.fAscii], TRUE

        invoke  SendMessage, [.hwnd], WM_GETTEXT, esi, [ebx+TInMemoryFile.ptrBuffer]
        xor     eax, eax
        clc
        return

.errorgettext:
        mov     eax, 1
        clc
        return

;----------------------------------------------------
onmessage CEM_LOADFILE

        mov     eax, [.wparam]
        test    eax, eax
        jnz     .loadit

        cmp     [edi+TOpenFile.fNeverSaved], 0
        jne     .skiploading

; Load the file in ASMEDIT window.
        mov     eax, [edi+TOpenFile.hFileName]

.loadit:
        stdcall StrPtr, eax
        stdcall LoadBinaryFile, eax
        jc      .endload ; eax contains the error code

        push    eax
        invoke  SendMessage, [.hwnd], WM_SETTEXT, 0, eax
        stdcall FreeMem ;  address from the stack.

        stdcall AddToMRUList, [ptrMRUFiles], [edi+TOpenFile.hFileName]

.skiploading:
        invoke  GetParent, [.hwnd]
        invoke  SendMessage, eax, EHM_FILECHANGED, edi, 0   ; Set the parameters of the tab.
        xor     eax, eax

.endload:
        clc
        return

        clc
        return

;---------------------------------------------------------------------
onmessage _CEM_CHANGENOTIFY

        invoke  GetParent, [.hwnd]
        invoke  SendMessage, eax, EHM_FILECHANGED, edi, 0
        clc
        return

;---------------------------------------------------------------------
onmessage _CEM_FOCUSFILE

        invoke  GetParent, [.hwnd]
        invoke  SendMessage, eax, EHM_FOCUSWINDOW, [.hwnd], 0
        clc
        return

;---------------------------------------------------------------------
onmessage _CEM_CANACTION

        mov     eax, [.lparam]
        cmp     eax, WM_CUT
        je      .checkcopycut
        cmp     eax, WM_COPY
        je      .checkcopycut
        cmp     eax, WM_CLEAR
        je      .checkcopycut
        cmp     eax, WM_UNDO
        je      .checkundo
        cmp     eax, EM_UNDO
        je      .checkundo
        cmp     eax, WM_PASTE
        je      .checkpaste
        xor     eax, eax
        clc
        return

.checkcopycut:
        lea     esi, [.aepos]
        invoke  SendMessage, [.hwnd], AEM_GETPOS, esi, 0
        mov     eax, [esi+AEPOS.caretPosition]
        mov     ecx, [esi+AEPOS.caretLine]
        cmp     [esi+AEPOS.selectionPosition], eax
        jne     .can_copy
        cmp     [esi+AEPOS.selectionLine], ecx
        jne     .can_copy
        xor     eax, eax
        clc
        return

.can_copy:
        xor     eax, eax
        inc     eax
        clc
        return

.checkundo:
        invoke  SendMessage, [.hwnd], EM_CANUNDO, 0, 0
        clc
        return

.checkpaste:
        invoke  IsClipboardFormatAvailable, CF_TEXT
        clc
        return

;---------------------------------------------------------------------
onmessage _CEM_EDITACTION

        invoke  SendMessage, [.hwnd], [.lparam], 0, 0
        clc
        return

;---------------------------------------------------------------------
onmessage _CEM_GETMODIFIED

        invoke  SendMessage, [.hwnd], AEM_GETMODIFIED, 0, 0
        clc
        return

;---------------------------------------------------------------------
onmessage _CEM_SETMODIFIED

        invoke  SendMessage, [.hwnd], AEM_SETMODIFIED, [.wparam], [.lparam]
        clc
        return

;---------------------------------------------------------------------
onmessage _CEM_GETFILE

        mov     eax, edi
        clc
        return

;---------------------------------------------------------------------
onmessage _CEM_GETASMEDIT

        mov     eax, [.hwnd]
        clc
        return

;---------------------------------------------------------------------
onmessage WM_KILLFOCUS

        stdcall ClosePopup, [hPAHint], [.hwnd], [.wparam]
        stdcall ClosePopup, [hCCList], [.hwnd], [.wparam]
        jmp     .ondefault


;---------------------------------------------------------------------
onmessage CCM_UPDATEPAPOS
        stdcall UpdatePAHint
        clc
        return

onmessage CCM_UPDATECCPOS
        stdcall UpdateCCPosition
        clc
        return

;-----------------------------------------------------------------------
onmessage WM_LBUTTONDOWN
onmessage WM_RBUTTONDOWN

        stdcall ClosePopup, [hCCList], 0, 0
        stdcall ClosePopup, [hPAHint], 0, 0

        jmp     .ondefault

;-----------------------------------------------------------------------
onmessage WM_VSCROLL
onmessage WM_HSCROLL
        stdcall ClosePopup, [hCCList], 0, 0
        stdcall ClosePopup, [hPAHint], 0, 0
        jmp     .ondefault


;-----------------------------------------------------------------------
onmessage WM_DROPFILES
locals
  .filename rb 1024
  .drop     POINT
  .incl     dd ?
endl

iglobal
  cIncludeCommand db 'include "', 0
  cIncludeEnd     db '"',0
  cIncludeEnd2    db $0d, $0a, 0
endg

        lea     eax, [.drop]
        invoke  DragQueryPoint, [.wparam], eax

        mov     ebx, [.drop.y]
        shl     ebx, 16
        mov     bx, word [.drop.x]
        invoke  SendMessage, [.hwnd], WM_LBUTTONDOWN, 0, ebx
        invoke  SendMessage, [.hwnd], WM_LBUTTONUP, 0, ebx

        lea     esi, [.aepos]
        invoke  SendMessage, [.hwnd], AEM_GETPOS, esi, 0
        mov     [esi+AEPOS.caretPosition], 1
        mov     [esi+AEPOS.selectionPosition], 1
        invoke  SendMessage, [.hwnd], AEM_SETPOS, esi, 0

        invoke  SendMessage, [.hwnd], AEM_GETMODE, 0, 0
        push    eax

        invoke  SendMessage, [.hwnd], AEM_SETMODE, 0, 0

        lea     esi, [.filename]

        invoke  DragQueryFile, [.wparam], -1, esi, 1024
        mov     ebx, eax
        xor     edi, edi

        stdcall StrNew
        mov     [.incl], eax

.opendroploop:
        invoke  DragQueryFile, [.wparam], edi, esi, 1024

        stdcall StrCat, [.incl], cIncludeCommand

        stdcall _RelativeToFinc, esi
        stdcall StrCat, [.incl], eax
        stdcall StrDel, eax

        stdcall StrCat, [.incl], cIncludeEnd

        inc     edi
        dec     ebx
        jnz     .opendroploop

        invoke  DragFinish, [.wparam]

        stdcall StrCat, [.incl], cIncludeEnd2
        stdcall StrPtr, [.incl]
        invoke  SendMessage, [.hwnd], EM_REPLACESEL, TRUE, eax
        stdcall StrDel, [.incl]

        pop     eax
        invoke  SendMessage, [.hwnd], AEM_SETMODE, eax, 0

        lea     esi, [.aepos]
        invoke  SendMessage, [.hwnd], AEM_GETPOS, esi, 0
        mov     eax, [esi+AEPOS.selectionPosition]
        mov     ecx, [esi+AEPOS.selectionLine]
        mov     [esi+AEPOS.caretPosition], eax
        mov     [esi+AEPOS.caretLine], ecx
        invoke  SendMessage, [.hwnd], AEM_SETPOS, esi, 0

        xor     eax, eax
        clc
        return


;-----------------------------------------------------------------------
onmessage WM_KEYDOWN
;        invoke  PostMessage, [.hwnd], CCM_UPDATEPAPOS, 0, 0

        cmp     [hCCList], 0
        je      .maybeopen

        mov     ebx, [.wparam]
        call    JumpTo
        MessageList                                      \
          VK_DOWN,   .sendtocc,                          \
          VK_UP,     .sendtocc,                          \
          VK_PRIOR,  .sendtocc,                          \
          VK_NEXT,   .sendtocc,                          \
          VK_HOME,   .sendtocc,                          \
          VK_END,    .sendtocc,                          \
          VK_LEFT,   .closekey,                          \
          VK_RIGHT,  .closekey,                          \
          VK_RETURN, .inserttext

        invoke MapVirtualKey, [.wparam], 2
        test   eax, eax
        jz     .ondefault

        cmp     al, ' '
        je      .closekey

        push    eax

        cmp     al, '-'
        jne     @f

        invoke  GetKeyState, VK_SHIFT
        test    ax, ax
        jns     @f

        mov     dword [esp], '_'

@@:
        pop     eax

        call    AsmEdit.recognize_character
        jnc     .endchar

        call    .insert_text_proc

        cmp     [.wparam], $bc
        jne     .maybedot

        invoke  GetKeyState, VK_SHIFT
        test    eax, $8000
        jnz     .closekey

        stdcall CreatePAHint, [.hwnd]
        jmp     .closekey

.maybedot:
        cmp     [.wparam], $be
        jne     .closekey

        invoke  GetKeyState, VK_SHIFT
        test    eax, $8000
        jnz     .closekey

.endchar:
        invoke  PostMessage, [.hwnd], CCM_UPDATECCPOS, 0, 0
        stc
        return

.closekey:
        invoke  PostMessage, [hCCList], WM_CLOSE, 0, 0
        stc
        return

.sendtocc:
        invoke  SendMessage, [hCCList], [.wmsg], [.wparam], [.lparam]
        clc
        return

.inserttext:
        call    .insert_text_proc
        invoke  PostMessage, [hCCList], WM_CLOSE, 0, 0
        clc
        return

;----------------------------------------------------------------------
.insert_text_proc:

        stdcall GetCCListSelected
        jc      .finishinsert

        lea     eax, [.str]
        invoke  SendMessage, [.hwnd], AEM_GETWORDATCARET, $ffff0100, eax

        mov     [.aepos.selectionLine], 0
        mov     [.aepos.caretLine], 0
        movzx   ecx, ax
        shr     eax, 16

        inc     ecx
        inc     eax

        mov     [.aepos.selectionPosition], eax
        mov     [.aepos.caretPosition], ecx
        lea     eax, [.aepos]
        invoke  SendMessage, [.hwnd], AEM_SETPOS, eax, 0
        invoke  SendMessage, [.hwnd], EM_REPLACESEL, FALSE, ebx

        lea     ebx, [.aepos]
        invoke  SendMessage, [.hwnd], AEM_GETPOS, ebx, 0
        push    [.aepos.selectionPosition] [.aepos.selectionLine]
        pop     [.aepos.caretLine] [.aepos.caretPosition]
        invoke  SendMessage, [.hwnd], AEM_SETPOS, ebx, 0

.finishinsert:
        retn


;-------------------------------------------------------------------

.maybeopen:
        mov     eax, [.wparam]
        cmp     eax, $be  ; point
        je      .point

        cmp     eax, VK_SPACE
        je      .space

        cmp     eax, 'Q'
        je      .ctrlQ

        cmp     eax, $bc
        jne     .ondefault

        invoke  GetKeyState, VK_SHIFT
        test    eax, $8000
        jnz     .ondefault

; open procedure arguments hint (hPAHint) window
.openPA:
        stdcall CreatePAHint, [.hwnd]
        stc
        return

.ctrlQ:
        invoke  GetKeyState, VK_CONTROL
        test    eax, $8000
        jnz     .openPA
        stc
        return

.space:
        invoke  GetKeyState, VK_CONTROL
        test    eax, $8000
        jz      .ondefault
        jmp     .doit

.point:
        invoke  GetKeyState, VK_SHIFT
        test    eax, $8000
        jnz     .ondefault

        lea     esi, [.str]
        invoke  SendMessage, [.hwnd], AEM_GETWORDATCARET, $ffff0100, esi

        cmp     byte [esi], 0
        je      .ondefault

.doit:
        stdcall CreateCCList, [.hwnd]
        invoke  PostMessage, [.hwnd], CCM_UPDATECCPOS, 0, 0

        cmp     [.wparam], $be
        je      .ondefault

        clc
        return

endwp




;-----------------------------------------------------------------------

proc OnGotoDefinition, .wparam, .lparam
.str rb 512
begin
        mov     edi, [ptrPreprocessed]
        test    edi, edi
        jz      .missing_info

        invoke  SendMessage, [hEditorsHost], EHM_GETCURRENTASMEDIT, 0, 0
        test    eax, eax
        jz      .finish

        lea     esi, [.str]
        invoke  SendMessage, eax, AEM_GETWORDATCARET, 512, esi

        cmp     byte [esi], 0
        je      .finish

        stdcall SearchTreeExact, esi, ptrLabels
        test    eax, eax
        jz      .finish

        test    [eax+TLabel.flags], lfDefined
        jz      .not_defined

        mov     eax, [eax+TLabel.pXRefArray]
        add     edi, [eax+TArray.array+TXRefInfo.ofsSource]         ; address of the preprocessed line where defined

.trytogo:
        mov     eax, [edi+TIntLine.LineNumber]
        test    eax, eax
        js      .frommacro

        mov     ecx, [edi+TIntLine.ofsGenerator]
        add     ecx, [ptrPreprocessed]
        cmp     ecx, [ptrPreprocessed]
        jne     .pos

        push    eax
        stdcall StrPtr, [hStrFileName]
        mov     ecx, eax
        pop     eax

.pos:
        stdcall PosEditor, ecx, eax

.finish:
        return

.missing_info:
        invoke  MessageBox, [hEditorsHost], cErrorNotCompiled, 0, MB_OK or MB_ICONINFORMATION
        jmp     .finish


.not_defined:
        invoke  MessageBox, [hEditorsHost], cErrorNotDefined, 0, MB_OK or MB_ICONINFORMATION
        jmp     .finish


.frommacro:
; the name of the macro
        mov   ecx, [edi+TIntLine.ofsGenerator]
        add   ecx, [ptrPreprocessed]
        cmp   word [ecx+1], '__'          ; internal macro definition... for example __UGlobalBlock or __IGlobalBlock
        je    .searchdown

        mov   edi, [edi+TIntLine.ofsSourceLine]
        add   edi, [ptrPreprocessed]
        jmp   .trytogo

.searchdown:
        mov     edi, [edi+TIntLine.ofsMacroLine]
        add     edi, [ptrPreprocessed]
        jmp     .trytogo
endp



proc ClosePopup, .hpopup, .host, .wparam
begin
        mov     eax, [.wparam]
        cmp     [.hpopup], 0
        je      .exit

        cmp     eax, [.hpopup]
        je      .exit

        cmp     eax, [.host]
        je      .exit

        invoke  PostMessage, [.hpopup], WM_CLOSE, 0, 0

.exit:
        return
endp